unit Visibles.TrimmingLabel;

interface

uses
  System.ComponentModel,
  System.Drawing,
  System.Windows.Forms;

type
  TTrimmingLabel = class;
  [ToolboxBitmapAttribute(TypeOf(TTrimmingLabel),'TTrimmingLabel.bmp')]
  TTrimmingLabel = class(System.Windows.Forms.Label)
  {$REGION 'Designer Managed Code'}
  strict private
    /// <summary>
    /// Required designer variable.
    /// </summary>
    Components: System.ComponentModel.Container;
    /// <summary>
    /// Required method for Designer support - do not modify
    /// the contents of this method with the code editor.
    /// </summary>
    procedure InitializeComponent;
  {$ENDREGION}
  strict private
    FTrimming: StringTrimming;
  strict protected
    procedure Initialize; virtual;
    procedure Dispose(Disposing: Boolean); override;
    function StringFormatForAlignment(Align: ContentAlignment): StringFormat;
    function CreateStringFormat: StringFormat;
    procedure OnPaint(e: PaintEventArgs); override;
  public
    procedure set_Trimming(const Value: StringTrimming);
    constructor Create; overload;
    constructor Create(Container: System.ComponentModel.IContainer); overload;
  published
    [Description('Determines the Text trimming mode'), Category('Appearance')]
    property Trimming: StringTrimming read FTrimming write set_Trimming default StringTrimming.None;
  end;

{$R 'Visibles.TTrimmingLabel.bmp'}

implementation

uses
  System.Drawing.Text;

{$REGION 'Windows Form Designer generated code'}
/// <summary>
/// Required method for Designer support - do not modify
/// the contents of this method with the code editor.
/// </summary>
procedure TTrimmingLabel.InitializeComponent;
begin
  Self.Components := System.ComponentModel.Container.Create;
end;
{$ENDREGION}

procedure TTrimmingLabel.Initialize;
begin
  Self.SetStyle(ControlStyles.Selectable,false);
  FTrimming := StringTrimming.None;
end;

constructor TTrimmingLabel.Create;
begin
  inherited Create;
  Initialize;
  InitializeComponent;
end;

constructor TTrimmingLabel.Create(Container: System.ComponentModel.IContainer);
begin
  inherited Create;
  Initialize;
  Container.Add(Self);
  InitializeComponent;
end;

procedure TTrimmingLabel.Dispose(Disposing: Boolean);
begin
  if Disposing then begin
    if Components <> nil then
      Components.Dispose();
  end;
  inherited Dispose(Disposing);
end;

function TTrimmingLabel.StringFormatForAlignment(Align: ContentAlignment): StringFormat;
begin
  Result := StringFormat.Create;
  case Align of
    ContentAlignment.BottomRight,ContentAlignment.MiddleRight,ContentAlignment.TopRight: Result.Alignment := StringAlignment.Far;
    ContentAlignment.BottomCenter,ContentAlignment.MiddleCenter,ContentAlignment.TopCenter: Result.Alignment := StringAlignment.Center;
    ContentAlignment.BottomLeft,ContentAlignment.MiddleLeft,ContentAlignment.TopLeft: Result.Alignment := StringAlignment.Near;
  end;
  case Align of
    ContentAlignment.BottomRight,ContentAlignment.BottomCenter,ContentAlignment.BottomLeft: Result.LineAlignment := StringAlignment.Far;
    ContentAlignment.MiddleRight,ContentAlignment.MiddleCenter,ContentAlignment.MiddleLeft: Result.LineAlignment := StringAlignment.Center;
    ContentAlignment.TopRight,ContentAlignment.TopCenter,ContentAlignment.TopLeft: Result.LineAlignment := StringAlignment.Near;
  end;
end;

function TTrimmingLabel.CreateStringFormat: StringFormat;
begin
  Result := StringFormatForAlignment(TextAlign);
  if RightToLeft = System.Windows.Forms.RightToLeft.Yes then
    Result.FormatFlags := (Result.FormatFlags or StringFormatFlags.DirectionRightToLeft);
  if not self.UseMnemonic then
    Result.HotkeyPrefix := HotkeyPrefix.None
  else if ShowKeyboardCues then
    Result.HotkeyPrefix := HotkeyPrefix.Show
  else
    Result.HotkeyPrefix := HotkeyPrefix.Hide;
  if AutoSize then
    Result.FormatFlags := (Result.FormatFlags or StringFormatFlags.MeasureTrailingSpaces);
  Result.Trimming := FTrimming;
  if FTrimming <> StringTrimming.None then
    Result.FormatFlags := (Result.FormatFlags or StringFormatFlags.NoWrap);
end;

procedure TTrimmingLabel.OnPaint(e: PaintEventArgs);
var
  AFormat: StringFormat;
  ABrush: Brush;
begin
  if Assigned(Image) then
    DrawImage(e.Graphics, Image, ClientRectangle, RtlTranslateAlignment(ImageAlign));
  AFormat := CreateStringFormat;
  if Enabled then begin
    ABrush := SolidBrush.Create(e.Graphics.GetNearestColor(ForeColor));
    e.Graphics.DrawString(Text, Font, ABrush, ClientRectangle, AFormat);
    ABrush.Dispose;
  end
  else
    ControlPaint.DrawStringDisabled(e.Graphics,Text,Font,e.Graphics.GetNearestColor(BackColor),ClientRectangle,AFormat);
  AFormat.Dispose;
end;

procedure TTrimmingLabel.set_Trimming(const Value: StringTrimming);
begin
  FTrimming := Value;
  Invalidate;
end;

end.


